# Copyright (c) 2018 Arista Networks, Inc.  All rights reserved.
# Arista Networks, Inc. Confidential and Proprietary.
'''
Bodega2GB ReloadPolicy

This ReloadPolicy plugin is used to safeguard Bodega 2GB flash systems from loading
into a non 2GB flavor image.
'''
import os.path
import re

# Cloverdale is only supported by EOS-2GB
sids2GB = [ 'cloverdale', ]

def is2GBBodegaSystem():
   # Check /etc/prefdl to see if the SID is only supported by 2GB
   # This covers all SID variants such as CloverdaleSsd, etc
   preFdlPath = '/etc/prefdl'
   if os.path.isfile( preFdlPath ):
      with open( preFdlPath ) as prefdl:
         prefdlStr = prefdl.read()
         sidMatch = re.search( r'^SID:\s*(.+)\s*$', prefdlStr,
                               re.MULTILINE | re.IGNORECASE )

         if sidMatch:
            sid = sidMatch.group( 1 ).lower()
            return any( sid.startswith( s ) for s in sids2GB )

   # Return false if prefdl information is missing
   return False

def check2GBImage( ctx ):
   imageFlavor = ctx.nextVersion.swiFlavor()
   if not imageFlavor or "2GB" not in imageFlavor:
      errMsg = ( "This platform is only supported in 2GB swi images. "
                 "Please update your image in /mnt/flash to a 2GB swi image "
                 "such as EOS-2GB.swi" )
      ctx.addError( errMsg )
      return False
   return True

def Plugin( ctx ):
   categories = [ 'ASU', 'ASU+', 'General' ]

   if is2GBBodegaSystem():
      ctx.addPolicy( check2GBImage, categories )
