#!/bin/sh

# This file maps a version of a product to a psp file containing support for
# that product. An instance of a product is defined as the platform string returned
# by the 'onie-sysinfo' command.  This string is of the form:
#
#       <arch>_<platform>_r<revision>
#
# All spaces and dashes (-) are replaced with underscores

# ONIE platform string => PSP filename
platformPspMap=$(cat <<EOF
x86_64_cel_smallstone_xp_r0=DqarQ.psp
x86_64_cel_redstone_xp_r0=DqarS.psp
x86_64_cel_questone_r0=DqarY.psp
x86_64_cel_seastone_r0=DqarC.psp
x86_64_accton_as5712_54x_r0=Jakku.psp
x86_64_dell_s6000_s1220_r0=NabooQ.psp
x86_64_dell_s4000_c2338_r0=NabooS.psp
x86_64_quanta_ly6_rangeley_r0=TakodanaQ.psp
x86_64_quanta_ly8_rangeley_r0=TakodanaS.psp
x86_64_quanta_ix1_rangeley_r0=TakodanaC.psp
x86_64_quanta_ix2_rangeley_r0=TakodanaY.psp
x86_64_quanta_ix7_rglbmc_r0=TakodanaT.psp
x86_64_quanta_ix8_rglbmc_r0=TakodanaZ.psp
x86_64_accton_wedge100_32x_r0=JakkuC.psp
x86_64_accton_wedge100s_32x_r0=JakkuC.psp
x86_64_facebook_wedge100_r0=JakkuC.psp
x86_64_accton_as7726_32x_r0=JakkuT.psp
x86_64_accton_as7326_56x_r0=JakkuZ.psp
EOF
)

for platformPsp in $platformPspMap; do
   eval $platformPsp
done

cmd="onie-sysinfo"

if [ "$1" == "--platforms" ]; then
   echo "$platformPspMap" | cut -d'=' -f1
elif [ "$1" == "--all" ]; then
   echo "$platformPspMap"
elif [ "$1" != "" ]; then
   ONIE_PLATFORM=$1
elif hash $cmd 2>/dev/null; then
   ONIE_PLATFORM=$($cmd)
fi

if [ -n "${ONIE_PLATFORM}" ]; then
   ONIE_PLATFORM=$(printf $ONIE_PLATFORM | tr -c '[0-9a-zA-Z_]' '_')
   pspMatch=$(eval "echo \$$ONIE_PLATFORM")
   if [ -n "$pspMatch" ]; then
      echo $ONIE_PLATFORM $pspMatch
   fi
fi
