# Copyright (c) 2018 Arista Networks, Inc.  All rights reserved.
# Arista Networks, Inc. Confidential and Proprietary.

# N.B. If you edit this file you must regenerate and replace
# the associated AsuPatch patch RPM. For more information see
# /src/AsuPatch/packages/README.

import Tracing
import AsuPStore
import Tac

__defaultTraceHandle__ = Tracing.Handle( "Dot1xAsu" )
t0 = Tracing.trace0
t1 = Tracing.trace1

class Dot1xPStoreEventHandler( AsuPStore.PStoreEventHandler ):
   def __init__( self, stores ):
      self.stores = stores
      self.funcs = { 'status' : self.storeStatus,
                     'hostTable' : self.storeHostTable,
                     'brConfig' : self.storeBrConfig,
                     'sessionIdStore' : self.storeSessionIdStore,
                     'allowedVlans' : self.storeAllowedVlans,
                     'vlanIdSet' : self.storeVlanIdSet,
                     'intfForwardingModel' : self.storeIntfForwardingModel
                     }
      super( Dot1xPStoreEventHandler, self ).__init__()
   
   def save( self, pStoreIO ):
      # Save TAC collection as dictionary in string format
      for k in self.getKeys():
         t0( 'saving', k )
         pyCol = self.funcs[k]()
         pStoreIO.set( k, pyCol )
   
   def getKeys( self ):
      return [ 'status', 'hostTable', 'brConfig', 'sessionIdStore', 'allowedVlans',
               'vlanIdSet', 'intfForwardingModel' ]

   def getSupportedKeys( self ):
      return self.getKeys()

   def hitlessReloadSupported( self ):
      t0( self.__class__.__name__ )
      return ( [], [] )

   def storeStatus( self ):
      t0( self.__class__.__name__, "store Dot1x Status." )
      status = self.stores[ 'status' ]
      pyCol = {}
      for intf, intfStatus in status.dot1xIntfStatus.iteritems():
         # Saving all the relevant attributes in Dot1xIntfStatus entity
         # Refer to AID5417 on more details on why do we need to store only the
         # attributes mentioned below:
         info = pyCol.setdefault( intf, {} )
         info[ 'portAuthState' ] = intfStatus.portAuthState
         info[ 'dynamicVlanId' ] = intfStatus.dynamicVlanId
         info[ 'hostMode' ] = intfStatus.hostMode
         info[ 'mbaConfigured' ] = intfStatus.mbaConfigured
         info[ 'reauthPeriod' ] = intfStatus.reauthPeriod
         info[ 'reauthBehavior' ] = intfStatus.reauthBehavior
         info[ 'lastManualReauthRequestTime' ] = \
               intfStatus.lastManualReauthRequestTime

         trunkDynamicAllowedVlans = info.setdefault( 'trunkDynamicAllowedVlans',
                                                     {} )
         for vlan in intfStatus.trunkDynamicAllowedVlan:
            trunkDynamicAllowedVlans[ vlan ] = True
         
         authSupplicantCol = info.setdefault( 'authSupplicant', {} )
         for mac in intfStatus.authSupplicant:
            authSupplicantCol[ mac ] = True
         
         dot1xAuthSupplicantCol = info.setdefault( 'dot1xAuthSupplicant', {} )
         for mac in intfStatus.dot1xAuthSupplicant:
            dot1xAuthSupplicantCol[ mac ] = True

         supplicantCol = info.setdefault( 'supplicant', {} )
         for mac, supplicant in intfStatus.supplicant.iteritems():
            suppInfo = supplicantCol.setdefault( mac, {} )
            suppInfo[ 'authStage' ] = supplicant.authStage
            suppInfo[ 'identity' ] = supplicant.identity
            suppInfo[ 'vlan' ]  = supplicant.vlan
            suppInfo[ 'vTag' ] = supplicant.vTag
            suppInfo[ 'taggedSupplicant' ] = supplicant.taggedSupplicant
            suppInfo[ 'supplicantTimeoutCounter' ] = \
                  supplicant.supplicantTimeoutCounter
            suppInfo[ 'supplicantAuthFailCounter' ] = \
                  supplicant.supplicantAuthFailCounter
            suppInfo[ 'supplicantDot1xCapable' ] = supplicant.supplicantDot1xCapable
            # Tac.endOfTime (inf) value is not corrctly handled by json c parser.
            if supplicant.lastAuthSuccessTime != Tac.endOfTime:
               suppInfo[ 'lastAuthSuccessTime' ] = supplicant.lastAuthSuccessTime
            suppInfo[ 'lastActivityTime' ] = supplicant.lastActivityTime
            suppInfo[ 'authSuccessTime' ] = supplicant.authSuccessTime
            suppInfo[ 'sessionTimeout' ] = supplicant.sessionTimeout
            suppInfo[ 'authenticating' ] = supplicant.authenticating
            suppInfo[ 'mbaSupplicant' ] = supplicant.mbaSupplicant
            # EapKey related info
            suppInfo[ 'dot1xType' ] = supplicant.eapKey.dot1xType
            suppInfo[ 'masterSessionKey' ] = supplicant.eapKey.masterSessionKey
            suppInfo[ 'eapSessionId' ] = supplicant.eapKey.eapSessionId
            suppInfo[ 'peerMacAddr' ] = supplicant.eapKey.peerMacAddr
            suppInfo[ 'myMacAddr' ] = supplicant.eapKey.myMacAddr

      return pyCol

   def storeHostTable( self ):
      t0( self.__class__.__name__, "store Dot1x HostTable." )
      hostTable = self.stores[ 'hostTable' ]
      pyCol = {}
      for mac, hostEntry in hostTable.hostEntry.iteritems():
         info = pyCol.setdefault( mac, {} )
         info[ 'vlanId' ] = hostEntry.vlanId
         info[ 'intfId' ] = hostEntry.intfId
         info[ 'mbaHost' ] = hostEntry.mbaHost

      return pyCol

   def storeSessionIdStore( self ): 
      t0( self.__class__.__name__, "store Dot1x SessionId store." )
      sessionIdStore = self.stores[ 'sessionIdStore' ]
      pyCol = {}
      pyCol[ 'acctSessionUniqueId' ] = sessionIdStore.acctSessionUniqueId
      acctSessionIdCol = pyCol.setdefault( 'acctSessionId', {} )
      for key, value in sessionIdStore.acctSessionId.iteritems():
         acctSessionIdCol[ key ] = value
      return pyCol
   
   def storeBrConfig( self ):
      t0( self.__class__.__name__, "store Dot1x BridgingInput config." )
      brConfig = self.stores[ 'brConfig' ]
      pyCol = {}
      pyCol[ 'switchIntfConfigPriority' ] = brConfig.switchIntfConfigPriority
      pyCol[ 'nonDefaultVal' ] = brConfig.nonDefaultVal
      switchIntfConfigCol = pyCol.setdefault( 'switchIntfConfig', {} )
      for intfId, switchIntfConfig in brConfig.switchIntfConfig.iteritems():
         switchIntfConfigInfo = switchIntfConfigCol.setdefault( intfId, {} )
         switchIntfConfigInfo[ 'switchportMode' ] = switchIntfConfig.switchportMode
         switchIntfConfigInfo[ 'accessVlan' ] = switchIntfConfig.accessVlan
         switchIntfConfigInfo[ 'trunkNativeVlan' ] = \
               switchIntfConfig.trunkNativeVlan
         switchIntfConfigInfo[ 'trunkAllowedVlans' ] = \
               switchIntfConfig.trunkAllowedVlans
      return pyCol

   def storeAllowedVlans( self ):
      t0( self.__class__.__name__, "store Dot1x input dynamic allowed vlans." )
      allowedVlans = self.stores[ 'allowedVlans' ]
      pyCol = {}
      for intf, vlans in allowedVlans.vlans.iteritems():
         pyCol[ intf ] = vlans
      return pyCol
   
   def storeVlanIdSet( self ):
      t0( self.__class__.__name__, "store Dot1x input dynamic vlan set." )
      vlanIdSet = self.stores[ 'vlanIdSet' ]
      pyCol = {}
      pyCol[ 'vlans' ] = vlanIdSet.vlans
      return pyCol
   
   def storeIntfForwardingModel( self ):
      t0( self.__class__.__name__, "store Dot1x interface forwarding model." )
      intfForwardingModel = self.stores[ 'intfForwardingModel' ]
      pyCol = {}
      for intf, forwardingModelInput in \
            intfForwardingModel.forwardingModelInput.iteritems():
         pyCol[ intf ] = forwardingModelInput
      return pyCol

def Plugin( ctx ):
   featureName = 'Dot1x'

   if ctx.opcode() == 'GetSupportedKeys':
      ctx.registerAsuPStoreEventHandler( featureName,
             Dot1xPStoreEventHandler( None ) )
      return

   entityManager = ctx.entityManager()
   mg = entityManager.mountGroup()
   stores = {}

   status = mg.mount( 'dot1x/status', 'Dot1x::Status', 'r' )
   hostTable = mg.mount( 'dot1x/hostTable', 'Dot1x::HostTable', 'r' )
   brConfig = mg.mount( 'bridging/input/config/dot1x',
                        'Bridging::Input::Config', 'r' )
   sessionIdStore = mg.mount( 'dot1x/sessionIdStore',
                              'Dot1x::AaaSessionIdStore', 'r' )
   allowedVlans = mg.mount( 'bridging/input/dynvlan/allowedvlan/dot1x',
                            'Bridging::Input::AllowedVlans', 'r' )
   vlanIdSet = mg.mount( 'bridging/input/dynvlan/vlan/dot1x',
                         'Bridging::Input::VlanIdSet', 'r' )
   intfForwardingModel = mg.mount( 'interface/input/forwardingmodel/dot1x',
                                   'Interface::IntfForwardingModelInputDir', 'r' )
   mg.close( blocking=True )

   # Stuff we actually store for ASU2
   stores[ 'status' ] = status
   stores[ 'hostTable' ] = hostTable
   stores[ 'brConfig' ] = brConfig
   stores[ 'sessionIdStore' ] = sessionIdStore
   stores[ 'allowedVlans' ] = allowedVlans
   stores[ 'vlanIdSet' ] = vlanIdSet
   stores[ 'intfForwardingModel' ] = intfForwardingModel
   
   ctx.registerAsuPStoreEventHandler( featureName,
                                      Dot1xPStoreEventHandler( stores ) )
